@@ -81,6 +81,7 @@
 #include "dsda/settings.h"
 #include "dsda/signal_context.h"
 #include "dsda/time.h"
+#include "dsda/utility.h"
 
 void I_uSleep(unsigned long usecs)
 {
@@ -123,6 +124,8 @@ fixed_t I_GetTimeFrac (void)
   }
   else
   {
+    static fixed_t last_frac;
+    static int last_gametic;
     unsigned long long tic_time;
     const double tics_per_usec = TICRATE / 1000000.0f;
 
@@ -130,6 +133,14 @@ fixed_t I_GetTimeFrac (void)
 
     frac = (fixed_t) (tic_time * FRACUNIT * tics_per_usec);
     frac = BETWEEN(0, FRACUNIT, frac);
+
+    if (frac < last_frac && last_gametic == gametic)
+    {
+      frac = FRACUNIT;
+    }
+
+    last_frac = frac;
+    last_gametic = gametic;
   }
 
   return frac;
@@ -226,11 +237,15 @@ void I_SwitchToWindow(HWND hwnd)
   }
 }
 
-const char *I_DoomExeDir(void)
+const char *I_ConfigDir(void)
+{
+  return I_ExeDir();
+}
+
+const char *I_ExeDir(void)
 {
   extern char **dsda_argv;
 
-  static const char current_dir_dummy[] = {"."}; // proff - rem extra slash 8/21/03
   static char *base;
   if (!base)        // cache multiple requests
     {
@@ -246,7 +261,7 @@ const char *I_DoomExeDir(void)
         Z_Free(base);
         base = (char*)Z_Malloc(1024);
         if (!M_getcwd(base, 1024) || !M_WriteAccess(base))
-          strcpy(base, current_dir_dummy);
+          strcpy(base, ".");
       }
     }
   return base;
@@ -274,7 +289,12 @@ const char* I_GetTempDir(void)
 
 #elif defined(AMIGA)
 
-const char *I_DoomExeDir(void)
+const char *I_ConfigDir(void)
+{
+  return "PROGDIR:";
+}
+
+const char *I_ExeDir(void)
 {
   return "PROGDIR:";
 }
@@ -285,26 +305,60 @@ const char* I_GetTempDir(void)
 }
 
 #else
-// cph - V.Aguilar (5/30/99) suggested return ~/.lxdoom/, creating
-//  if non-existant
-// cph 2006/07/23 - give prboom+ its own dir
-static const char prboom_dir[] = {"/.dsda-doom"}; // Mead rem extra slash 8/21/03
 
-const char *I_DoomExeDir(void)
+const char *I_ConfigDir(void)
 {
   static char *base;
-  if (!base)        // cache multiple requests
+
+  if (!base)
   {
     char *home = M_getenv("HOME");
-    size_t len = strlen(home);
 
-    base = Z_Malloc(len + strlen(prboom_dir) + 1);
-    strcpy(base, home);
-    // I've had trouble with trailing slashes before...
-    if (base[len-1] == '/') base[len-1] = 0;
-    strcat(base, prboom_dir);
+    // First, try legacy directory.
+    base = dsda_ConcatDir(home, ".dsda-doom");
+    if (access(base, F_OK) != 0)
+    {
+      // Legacy directory is not accessible. Use XDG directory.
+      char *xdg_data_home;
+
+      Z_Free(base);
+
+      xdg_data_home = M_getenv("XDG_DATA_HOME");
+      if (xdg_data_home)
+        base = dsda_ConcatDir(xdg_data_home, "dsda-doom");
+      else
+        // $XDG_DATA_HOME should be $HOME/.local/share if not defined.
+        base = dsda_ConcatDir(home, ".local/share/dsda-doom");
+    }
+
     M_MakeDir(base, true); // Make sure it exists
   }
+
+  return base;
+}
+
+const char *I_ExeDir(void)
+{
+  extern char **dsda_argv;
+
+  static char *base;
+  if (!base)        // cache multiple requests
+    {
+      size_t len = strlen(*dsda_argv);
+      char *p = (base = (char*)Z_Malloc(len+1)) + len - 1;
+      strcpy(base,*dsda_argv);
+      while (p > base && *p!='/' && *p!='\\')
+        *p--=0;
+      if (*p=='/' || *p=='\\')
+        *p--=0;
+      if (strlen(base) < 2 || !M_WriteAccess(base))
+      {
+        Z_Free(base);
+        base = (char*)Z_Malloc(1024);
+        if (!M_getcwd(base, 1024) || !M_WriteAccess(base))
+          strcpy(base, ".");
+      }
+    }
   return base;
 }
 
@@ -366,9 +420,12 @@ char* I_FindFileInternal(const char* wfname, const char* ext, dboolean isStatic)
     const char *dir; // directory
     const char *sub; // subdirectory
     const char *env; // environment variable
-    const char *(*func)(void); // for I_DoomExeDir
+    const char *(*func)(void); // for functions that return the directory
   } search0[] = {
-    {NULL, NULL, NULL, I_DoomExeDir}, // config directory
+    {NULL, NULL, NULL, I_ExeDir}, // executable directory
+#ifndef _WIN32
+    {NULL, NULL, NULL, I_ConfigDir}, // config and autoload directory. on windows, this is the same as I_ExeDir
+#endif
     {NULL}, // current working directory
     {NULL, NULL, "DOOMWADDIR"}, // run-time $DOOMWADDIR
     {DOOMWADDIR}, // build-time configured DOOMWADDIR
