@@ -268,6 +268,11 @@ static int I_TranslateKey(SDL_Keysym* key)
 
 }
 
+dboolean I_WindowFocused(void)
+{
+  return window_focused;
+}
+
 /////////////////////////////////////////////////////////////////////////////////
 // Main input code
 
@@ -294,122 +299,128 @@ static void I_GetEvent(void)
   SDL_Event SDLEvent;
   SDL_Event *Event = &SDLEvent;
 
-while (SDL_PollEvent(Event))
-{
-  switch (Event->type) {
-  case SDL_KEYDOWN:
+  while (SDL_PollEvent(Event))
+  {
+    switch (Event->type) {
+      case SDL_KEYDOWN:
 #ifdef __APPLE__
-    if (Event->key.keysym.mod & KMOD_GUI)
-    {
-      // Switch windowed<->fullscreen if pressed <Command-F>
-      if (Event->key.keysym.sym == SDLK_f)
-      {
-        V_ToggleFullscreen();
-        break;
-      }
-    }
+        if (Event->key.keysym.mod & KMOD_GUI)
+        {
+          // Switch windowed<->fullscreen if pressed <Command-F>
+          if (Event->key.keysym.sym == SDLK_f)
+          {
+            V_ToggleFullscreen();
+            break;
+          }
+        }
 #else
-    if (Event->key.keysym.mod & KMOD_LALT)
-    {
-      // Prevent executing action on Alt-Tab
-      if (Event->key.keysym.sym == SDLK_TAB)
-      {
-        break;
-      }
-      // Switch windowed<->fullscreen if pressed Alt-Enter
-      else if (Event->key.keysym.sym == SDLK_RETURN)
-      {
-        V_ToggleFullscreen();
-        break;
-      }
-      // Immediately exit on Alt+F4 ("Boss Key")
-      else if (Event->key.keysym.sym == SDLK_F4)
-      {
-        I_SafeExit(0);
-        break;
-      }
-    }
+        if (Event->key.keysym.mod & KMOD_LALT)
+        {
+          // Prevent executing action on Alt-Tab
+          if (Event->key.keysym.sym == SDLK_TAB)
+          {
+            break;
+          }
+          // Switch windowed<->fullscreen if pressed Alt-Enter
+          else if (Event->key.keysym.sym == SDLK_RETURN)
+          {
+            V_ToggleFullscreen();
+            break;
+          }
+          // Immediately exit on Alt+F4 ("Boss Key")
+          else if (Event->key.keysym.sym == SDLK_F4)
+          {
+            I_SafeExit(0);
+            break;
+          }
+        }
 #endif
-    event.type = ev_keydown;
-    event.data1.i = I_TranslateKey(&Event->key.keysym);
-    D_PostEvent(&event);
-    break;
-
-  case SDL_KEYUP:
-  {
-    event.type = ev_keyup;
-    event.data1.i = I_TranslateKey(&Event->key.keysym);
-    D_PostEvent(&event);
-  }
-  break;
-
-  case SDL_MOUSEBUTTONDOWN:
-  case SDL_MOUSEBUTTONUP:
-  if (mouse_enabled && window_focused)
-  {
-    event.type = ev_mouse;
-    event.data1.i = I_SDLtoDoomMouseState(SDL_GetMouseState(NULL, NULL));
-    D_PostEvent(&event);
-  }
-  break;
-
-  case SDL_MOUSEWHEEL:
-  if (mouse_enabled && window_focused)
-  {
-    if (Event->wheel.y > 0)
-    {
-      event.data1.i = KEYD_MWHEELUP;
-
-      event.type = ev_keydown;
-      D_PostEvent(&event);
-
-      event.type = ev_keyup;
-      D_PostEvent(&event);
-    }
-    else if (Event->wheel.y < 0)
-    {
-      event.data1.i = KEYD_MWHEELDOWN;
-
-      event.type = ev_keydown;
-      D_PostEvent(&event);
-
-      event.type = ev_keyup;
-      D_PostEvent(&event);
-    }
-  }
-  break;
-
-  case SDL_TEXTINPUT:
-    event.type = ev_text;
-    event.text = Event->text.text;
-    D_PostEvent(&event);
-    break;
-
-  case SDL_WINDOWEVENT:
-    if (Event->window.windowID == windowid)
-    {
-      switch (Event->window.event)
-      {
-      case SDL_WINDOWEVENT_FOCUS_GAINED:
-      case SDL_WINDOWEVENT_FOCUS_LOST:
-        UpdateFocus();
+        event.type = ev_keydown;
+        event.data1.i = I_TranslateKey(&Event->key.keysym);
+        D_PostEvent(&event);
         break;
-      case SDL_WINDOWEVENT_SIZE_CHANGED:
-        ApplyWindowResize(Event);
+
+      case SDL_KEYUP:
+        {
+          event.type = ev_keyup;
+          event.data1.i = I_TranslateKey(&Event->key.keysym);
+          D_PostEvent(&event);
+        }
+        break;
+
+      case SDL_MOUSEBUTTONDOWN:
+      case SDL_MOUSEBUTTONUP:
+        if (mouse_enabled && window_focused)
+        {
+          event.type = ev_mouse;
+          event.data1.i = I_SDLtoDoomMouseState(SDL_GetMouseState(NULL, NULL));
+          D_PostEvent(&event);
+        }
+        break;
+
+      case SDL_MOUSEWHEEL:
+        if (mouse_enabled && window_focused)
+        {
+          if (Event->wheel.y > 0)
+          {
+            event.data1.i = KEYD_MWHEELUP;
+
+            event.type = ev_keydown;
+            D_PostEvent(&event);
+
+            event.type = ev_keyup;
+            D_PostEvent(&event);
+          }
+          else if (Event->wheel.y < 0)
+          {
+            event.data1.i = KEYD_MWHEELDOWN;
+
+            event.type = ev_keydown;
+            D_PostEvent(&event);
+
+            event.type = ev_keyup;
+            D_PostEvent(&event);
+          }
+        }
+        break;
+
+      case SDL_CONTROLLERBUTTONDOWN:
+      case SDL_CONTROLLERBUTTONUP:
+        if (dsda_AllowGameController())
+          dsda_PollGameControllerButtons();
+        break;
+
+      case SDL_TEXTINPUT:
+        event.type = ev_text;
+        event.text = Event->text.text;
+        D_PostEvent(&event);
+        break;
+
+      case SDL_WINDOWEVENT:
+        if (Event->window.windowID == windowid)
+        {
+          switch (Event->window.event)
+          {
+          case SDL_WINDOWEVENT_FOCUS_GAINED:
+          case SDL_WINDOWEVENT_FOCUS_LOST:
+            UpdateFocus();
+            break;
+          case SDL_WINDOWEVENT_SIZE_CHANGED:
+            ApplyWindowResize(Event);
+            break;
+          }
+        }
+        break;
+
+      case SDL_QUIT:
+        S_StartVoidSound(sfx_swtchn);
+        M_QuitDOOM(0);
+
+      default:
         break;
-      }
     }
-    break;
-
-  case SDL_QUIT:
-    S_StartVoidSound(sfx_swtchn);
-    M_QuitDOOM(0);
-
-  default:
-    break;
   }
 }
-}
 
 //
 // I_StartTic
@@ -433,6 +444,13 @@ void I_StartFrame (void)
 {
 }
 
+static void I_FlushMousePosition(void)
+{
+  int x, y;
+
+  SDL_GetRelativeMouseState(&x, &y);
+}
+
 void I_InitMouse(void)
 {
   static Uint8 empty_cursor_data = 0;
@@ -446,6 +464,8 @@ void I_InitMouse(void)
   cursors[0] = SDL_GetCursor();
   // Create an empty cursor
   cursors[1] = SDL_CreateCursor(&empty_cursor_data, &empty_cursor_data, 8, 1, 0, 0);
+
+  I_FlushMousePosition();
 }
 
 //
@@ -526,6 +546,34 @@ void I_ShutdownGraphics(void)
   DeactivateMouse();
 }
 
+static dboolean queue_frame_capture;
+static dboolean queue_screenshot;
+
+void I_QueueFrameCapture(void)
+{
+  queue_frame_capture = true;
+}
+
+void I_QueueScreenshot(void)
+{
+  queue_screenshot = true;
+}
+
+void I_HandleCapture(void)
+{
+  if (queue_frame_capture)
+  {
+    I_CaptureFrame();
+    queue_frame_capture = false;
+  }
+
+  if (queue_screenshot)
+  {
+    M_ScreenShot();
+    queue_screenshot = false;
+  }
+}
+
 //
 // I_FinishUpdate
 //
@@ -591,6 +639,8 @@ void I_FinishUpdate (void)
 
   SDL_RenderCopy(sdl_renderer, sdl_texture, &src_rect, NULL);
 
+  I_HandleCapture();
+
   // Draw!
   SDL_RenderPresent(sdl_renderer);
 }
@@ -1168,7 +1218,7 @@ void I_InitGraphics(void)
 
 void I_UpdateVideoMode(void)
 {
-  int init_flags = 0;
+  int init_flags = SDL_WINDOW_ALLOW_HIGHDPI;
   int screen_multiply;
   int actualheight;
   int render_vsync;
@@ -1214,7 +1264,7 @@ void I_UpdateVideoMode(void)
 
   // Initialize SDL with this graphics mode
   if (V_IsOpenGLMode()) {
-    init_flags = SDL_WINDOW_OPENGL;
+    init_flags |= SDL_WINDOW_OPENGL;
   }
 
   if (desired_fullscreen)
@@ -1255,7 +1305,7 @@ void I_UpdateVideoMode(void)
     sdl_window = SDL_CreateWindow(
       PACKAGE_NAME " " PACKAGE_VERSION,
       SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
-      SCREENWIDTH, SCREENHEIGHT,
+      SCREENWIDTH * screen_multiply, SCREENHEIGHT * screen_multiply,
       init_flags);
     sdl_glcontext = SDL_GL_CreateContext(sdl_window);
     SDL_SetWindowMinimumSize(sdl_window, SCREENWIDTH, SCREENHEIGHT);
@@ -1373,9 +1423,6 @@ void I_UpdateVideoMode(void)
   V_SetPalette(0);
   I_UploadNewPalette(0, true);
 
-  ST_SetResolution();
-  AM_SetResolution();
-
   if (V_IsOpenGLMode())
   {
     int temp;
@@ -1412,6 +1459,9 @@ void I_UpdateVideoMode(void)
     gld_Init(SCREENWIDTH, SCREENHEIGHT);
   }
 
+  ST_SetResolution();
+  AM_SetResolution();
+
   if (V_IsOpenGLMode())
   {
     M_ChangeFOV();
@@ -1552,15 +1602,10 @@ static void UpdateFocus(void)
   // after switching to OS and back
   if (desired_fullscreen && window_focused)
   {
-    // currentPaletteIndex?
-    if (st_palette < 0)
-      st_palette = 0;
-
-    V_SetPalette(st_palette);
+    V_TouchPalette();
   }
 
-  // Should the screen be grabbed?
-  //    screenvisible = (state & SDL_APPACTIVE) != 0;
+  S_ResetVolume();
 }
 
 void UpdateGrab(void)
