@@ -122,7 +122,6 @@ enum {
   comperr_passuse,
   comperr_hangsolid,
   comperr_blockmap,
-  comperr_freeaim,
 
   COMPERR_NUM
 };
@@ -351,6 +350,4 @@ extern int monkeys;
 
 extern int HelperThing;          // type of thing to use for helper
 
-extern dboolean forceOldBsp;
-
 #endif
