@@ -186,7 +186,7 @@ void dsda_InitSaveDir(void) {
   dsda_base_save_dir = dsda_DetectDirectory("DOOMSAVEDIR", dsda_arg_save);
 }
 
-static char* dsda_SaveDir(void) {
+char* dsda_SaveDir(void) {
   if (dsda_IntConfig(dsda_config_organized_saves)) {
     if (!dsda_wad_save_dir)
       dsda_wad_save_dir = dsda_DataDir();
@@ -283,3 +283,25 @@ int dsda_LastSaveSlot(void) {
 void dsda_ResetLastSaveSlot(void) {
   last_save_file_slot = -1;
 }
+
+void dsda_UpdateAutoSaves(void) {
+  static int automap = -1;
+  static int autoepisode = -1;
+
+  void M_AutoSave(void);
+
+  if (!dsda_IntConfig(dsda_config_auto_save) ||
+      gamestate != GS_LEVEL ||
+      gameaction != ga_nothing ||
+      demoplayback ||
+      demorecording)
+    return;
+
+  if (automap != gamemap || autoepisode != gameepisode) {
+    automap = gamemap;
+    autoepisode = gameepisode;
+
+    if (!leveltime)
+      M_AutoSave();
+  }
+}
