@@ -80,27 +80,12 @@ const byte *colrngs[CR_LIMIT];
 
 int usegamma;
 
-// [FG] translate between blood color value as per EE spec
-//      and actual color translation table index
-
-static const int bloodcolor[] = {
-  0,         // 0 - Red (normal)
-  CR_GRAY,   // 1 - Grey
-  CR_GREEN,  // 2 - Green
-  CR_BLUE,   // 3 - Blue
-  CR_YELLOW, // 4 - Yellow
-  CR_BLACK,  // 5 - Black
-  CR_PURPLE, // 6 - Purple
-  CR_WHITE,  // 7 - White
-  CR_ORANGE, // 8 - Orange
-};
-
 int V_BloodColor(int blood)
 {
-  if (blood < 0 || blood > 8)
-    blood = 0;
+  if (blood < 1 || blood > 8)
+    return 0;
 
-  return bloodcolor[blood];
+  return CR_BLOOD + blood - 1;
 }
 
 // haleyjd: DOSDoom-style single translucency lookup-up table
@@ -277,39 +262,29 @@ static void FUNC_V_CopyRect(int srcscrn, int destscrn,
 
 static void FUNC_V_FillFlat(int lump, int scrn, int x, int y, int width, int height, enum patch_translation_e flags)
 {
-  /* erase the entire screen to a tiled background */
   const byte *data;
-  int sx, sy, w, h;
-  int j, pitch;
+  byte *dest;
+  int sx, sy;
+  int pitch, src_x_offset, src_y_offset;
+  float ratio_x, ratio_y;
+  stretch_param_t* stretch;
 
-  lump += firstflat;
+  stretch = dsda_StretchParams(flags);
+  pitch = screens[scrn].pitch;
+  data = W_LumpByNum(lump + firstflat);
 
-  // killough 4/17/98:
-  data = W_LumpByNum(lump);
+  ratio_x = stretch->video->width / 320.f;
+  ratio_y = stretch->video->height / 200.f;
 
+  for (sy = y; sy < y + height; ++sy)
   {
-    const byte *src, *src_p;
-    byte *dest, *dest_p;
-    pitch = screens[scrn].pitch;
+    src_y_offset = 64 * ((int) ((sy - y) / ratio_y) % 64);
+    dest = screens[scrn].data + pitch * sy + x;
 
-    for (sy = y ; sy < y + height; sy += 64)
+    for (sx = x; sx < x + width; ++sx)
     {
-      h = (y + height - sy < 64 ? y + height - sy : 64);
-      dest = screens[scrn].data + pitch * sy + x;
-      src = data + 64 * ((sy - y) % 64);
-      for (sx = x; sx < x + width; sx += 64)
-      {
-        src_p = src;
-        dest_p = dest;
-        w = (x + width - sx < 64 ? x + width - sx : 64);
-        for (j = 0; j < h; j++)
-        {
-          memcpy (dest_p, src_p, w);
-          dest_p += pitch;
-          src_p += 64;
-        }
-        dest += 64;
-      }
+      src_x_offset = (int) ((sx - x) / ratio_x) % 64;
+      *dest++ = data[src_x_offset + src_y_offset];
     }
   }
 }
@@ -338,7 +313,7 @@ static void FUNC_V_FillPatch(int lump, int scrn, int x, int y, int width, int he
  */
 static void FUNC_V_DrawBackground(const char* flatname, int scrn)
 {
-  V_FillFlatName(flatname, scrn, 0, 0, SCREENWIDTH, SCREENHEIGHT, VPT_NONE);
+  V_FillFlatName(flatname, scrn, 0, 0, SCREENWIDTH, SCREENHEIGHT, VPT_STRETCH);
 }
 
 //
@@ -738,7 +713,7 @@ static void WRAP_gld_CopyRect(int srcscrn, int destscrn, int x, int y, int width
 }
 static void WRAP_gld_DrawBackground(const char *flatname, int n)
 {
-  gld_FillFlatName(flatname, 0, 0, SCREENWIDTH, SCREENHEIGHT, VPT_NONE);
+  gld_FillFlatName(flatname, 0, 0, SCREENWIDTH, SCREENHEIGHT, VPT_STRETCH);
 }
 static void WRAP_gld_FillFlat(int lump, int n, int x, int y, int width, int height, enum patch_translation_e flags)
 {
@@ -1298,9 +1273,12 @@ void SetRatio(int width, int height)
   tallscreen = (ratio_scale < ratio_multiplier);
   if (tallscreen)
   {
+    float ratio_quotient = (float)ratio_multiplier/ratio_scale;
+    float ratio_percentage = (ratio_quotient - 1) * 100.0;
+    psprite_offset = (int)(ratio_percentage*FRACUNIT);
+
     lprintf(LO_DEBUG, "SetRatio: tallscreen aspect recognized; flipping multiplier\n");
     swap(&ratio_multiplier, &ratio_scale);
-    psprite_offset = (int)(6.5*FRACUNIT);
   }
   else
   {
